﻿using System;
using System.Web;
using System.Security.Principal;

namespace Microsoft.WebServices.Security
{
    public class WebServiceAuthenticationEvent : EventArgs
    {
        private IPrincipal _principal;
        private readonly HttpContext _context;
        private string _user;
        private string _password;

        public WebServiceAuthenticationEvent(HttpContext context)
        {
            _context = context;
        }

        public WebServiceAuthenticationEvent(HttpContext context, string user, string password)
        {            
            if (user != null)
            {
                user = user.TrimStart('\\');
            }

            _context = context;
            _user = user;
            _password = password;
        }

        public HttpContext Context
        {
            get { return _context; }
        }

        public IPrincipal Principal
        {
            get { return _principal; }
            set { _principal = value; }
        }

        public void Authenticate()
        {
            GenericIdentity i = new GenericIdentity(User);
            this.Principal = new GenericPrincipal(i, new String[0]);
        }
        
        public void Authenticate(string[] roles)
        {
            GenericIdentity i = new GenericIdentity(User);
            this.Principal = new GenericPrincipal(i, roles);
        }
        
        public string User
        {
            get { return _user; }
            set { _user = value; }
        }
        
        public string Password
        {
            get { return _password; }
            set { _password = value; }
        }
        
        public bool HasCredentials
        {
            get
            {
                if ((_user == null) || (_password == null))
                    return false;
                return true;
            }
        }
    }
}